@@ -266,6 +266,7 @@ static void ScrollOneLineUp();
 static void ScrollOneLineDown();
 static void DeleteForwardChar();
 static void DeleteBackwardChar();
+static void DeleteBackwardNormal();
 static void DeleteForwardWord();
 static void DeleteBackwardWord();
 static void DeleteCurrentSelection();
@@ -313,7 +314,7 @@ XtActionsRec texteditActionsTable [] = {
   {"scroll-one-line-down", 	ScrollOneLineDown},
 /* delete bindings */
   {"delete-next-character", 	DeleteForwardChar},
-  {"delete-previous-character", DeleteBackwardChar},
+  {"delete-previous-character", DeleteBackwardNormal},
   {"delete-next-word", 		DeleteForwardWord},
   {"delete-previous-word", 	DeleteBackwardWord},
   {"delete-selection", 		DeleteCurrentSelection},
@@ -1505,6 +1506,22 @@ static void DeleteForwardChar(ctx, event)
   EndAction(ctx);
 }
 
+/*--------------------------------------------------------------------------+*/
+static void DeleteBackwardNormal(ctx, event)
+/*--------------------------------------------------------------------------+*/
+  XwTextEditWidget ctx;
+  XEvent *event;
+{
+  XwTextPosition next;
+
+  /* If there's a selection, delete it, otherwise, delete the character */
+
+  if (ctx->text.s.left != ctx->text.s.right)
+     DeleteCurrentSelection(ctx, event);
+  else
+     DeleteBackwardChar(ctx, event);
+}
+
 /*--------------------------------------------------------------------------+*/
 static void DeleteBackwardChar(ctx, event)
 /*--------------------------------------------------------------------------+*/
@@ -1515,8 +1532,8 @@ static void DeleteBackwardChar(ctx, event)
 
   StartAction(ctx, event);
   next = (*(ctx->text.source->scan))(
-            ctx->text.source, ctx->text.insertPos, XwstPositions, 
-	    XwsdLeft, 1, TRUE);
+         ctx->text.source, ctx->text.insertPos, XwstPositions, 
+	 XwsdLeft, 1, TRUE);
   DeleteOrKill(ctx, next, ctx->text.insertPos, FALSE);
   EndAction(ctx);
 }
@@ -2432,7 +2449,7 @@ XwTextEditClassRec XwtexteditClassRec = {
   {
     /* core fields */
     /* superclass       */      (WidgetClass) &XwprimitiveClassRec,
-    /* class_name       */      "TextEdit",
+    /* class_name       */      "XwTextEdit",
     /* widget_size      */      sizeof(XwTextEditRec),
     /* class_initialize */      ClassInitialize,
     /* class_part_init  */      NULL,
