@@ -50,7 +50,6 @@ extern short attachto;
 extern char  _STR[150], _STR2[250];
 extern stringpart *labelbuf;
 extern objectpair *pushlist;
-extern short pushes;
 
 #define RADFAC 0.0174532925199   /* pi / 180 */
 extern double atan2();
@@ -236,6 +235,26 @@ void charreport(labelptr curlabel)
    Wprintf(_STR);
 }
 
+/*----------------------------------------------------------------------*/
+/* See if a (pin) label has a copy (at least one) in this drawing.	*/
+/*----------------------------------------------------------------------*/
+
+labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
+{
+   genericptr *tgen;
+   labelptr tlab;
+
+   for (tgen = objectdata->plist; tgen < objectdata->plist + objectdata->parts; tgen++) {
+      if ((*tgen)->type == LABEL) {
+         tlab = TOLABEL(tgen);
+	 if (tlab->pin == False) continue;
+	 else if (tlab == curlabel) continue;  /* Don't count self! */
+         else if (!stringcomp(tlab->string, curstring)) return tlab;  
+      }
+   }
+   return NULL;
+}
+
 /*--------------------------------------------------------------*/
 /* Interpret string and add to current label.			*/
 /* 	keypressed is a KeySym					*/
@@ -248,18 +267,10 @@ void labeltext(int keypressed, char *clientdata)
    stringpart *curpos, *strptr;
    int locpos;
    Boolean do_redraw = False;
-
    short tmplength, tmpheight, cfont;
-#ifdef SCHEMA
-   labelptr symlabel;
-#endif
 
    curlabel = TOLABEL(EDITPART);
 
-#ifdef SCHEMA
-   symlabel = findotherpin(curlabel);
-#endif
-
    if (curlabel == NULL || textpos <= 0) {
       Wprintf("Error:  Bad label string");
       return;
@@ -365,6 +376,26 @@ void labeltext(int keypressed, char *clientdata)
       setdefaultfontmarks();
       setcolormark(areastruct.color);
       if (labelbuf != NULL) {
+
+#ifdef SCHEMA
+	 /* If the original label (before modification) is a pin in a	*/
+	 /* schematic/symbol with a matching symbol/schematic, and the	*/
+	 /* name is unique, change every pin in the matching symbol/	*/
+	 /* schematic to match the new text.				*/
+
+	 if ((areastruct.schemon == True) && (curlabel->pin != False) && 
+		(objectdata->symschem != NULL)) {
+	    if ((findlabelcopy(curlabel, labelbuf) == NULL)
+			&& (findlabelcopy(curlabel, curlabel->string) == NULL)) {
+	       changeotherpins(curlabel, labelbuf);
+	       if (objectdata->schemtype == SCHEMATIC)
+	          Wprintf("Changed corresponding pin in associated symbol");
+	       else
+	          Wprintf("Changed corresponding pin in associated schematic");
+	    }
+	 }
+#endif
+	
 	 freelabel(labelbuf);
 	 labelbuf = NULL;
 	 resolveparams(NORMINST);
@@ -466,10 +497,6 @@ void labeltext(int keypressed, char *clientdata)
       do_redraw = True;
    }
 
-#ifdef SCHEMA
-   if (symlabel != NULL) symlabel->string = curlabel->string;
-#endif
-
    /* Redraw the label */
 
    if (do_redraw) {
