@@ -1,12 +1,14 @@
 /*-------------------------------------------------------------------------*/
 /* fontfile.c --- Load font character and encoding defitions		   */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
    
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -18,6 +20,11 @@
 
 #include "xcircuit.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*-------------------------------------------------------------------------*/
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
@@ -30,9 +37,6 @@ extern fontinfo *fonts;
 extern Globaldata xobjs;
 extern short beeper;
 
-extern void setdefaultfontmarks();
-extern short findhelvetica();
-
 /*----------------------------------------------------------------------*/
 /* Find the file containing the encoding for the given font		*/
 /*----------------------------------------------------------------------*/
@@ -148,7 +152,7 @@ void loadfontfile(char *fontname)
    FILE *fd;
    char temp[250], commandstr[30], tempname[100];
    char *psname = NULL, *family = NULL;
-   char *cmdptr, *tmpptr;
+   char *cmdptr;
    int flags = 0;
    int i;
    float fontscale = 1.0;
