@@ -43,7 +43,6 @@ extern Globaldata xobjs;
 extern int *appcolors;
 extern short textpos, textend;
 extern objectpair *pushlist;
-extern short pushes;
 
 /*-----------------------*/
 /* Arithmetic functions  */
@@ -845,9 +844,88 @@ void calcextents(genericptr *bboxgen, short *llx, short *lly,
    }
 }
 
-/*-------------------------------------------------------------------------*/
-/* Bounding box calculation for a single generic element		   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Calculate the bounding box of an object instance			*/
+/*----------------------------------------------------------------------*/
+
+void objinstbbox(objinstptr obbox, XPoint *npoints)
+{
+   XPoint points[4];
+
+#ifdef SCHEMA
+   points[0].x = points[1].x = obbox->thisobject->lleft2.x;
+   points[1].y = points[2].y = obbox->thisobject->lleft2.y
+		+ obbox->thisobject->height2;
+   points[2].x = points[3].x = obbox->thisobject->lleft2.x
+		+ obbox->thisobject->width2;
+   points[0].y = points[3].y = obbox->thisobject->lleft2.y;
+#else
+   points[0].x = points[1].x = obbox->thisobject->lowerleft.x;
+   points[1].y = points[2].y = obbox->thisobject->lowerleft.y
+		+ obbox->thisobject->height;
+   points[2].x = points[3].x = obbox->thisobject->lowerleft.x
+		+ obbox->thisobject->width;
+   points[0].y = points[3].y = obbox->thisobject->lowerleft.y;
+#endif
+
+   UTransformPoints(points, npoints, 4, obbox->position,
+		 obbox->scale, obbox->rotation); 
+}
+
+/*----------------------------------------------------------------------*/
+/* Calculate the bounding box of a label				*/
+/*----------------------------------------------------------------------*/
+
+void labelbbox(labelptr labox, XPoint *npoints, objinstptr callinst)
+{
+   XPoint points[4];
+   TextExtents tmpext;
+   short j;
+
+   tmpext = ULength(labox->string, callinst, 0.0, 0, NULL);
+   points[0].x = points[1].x = (labox->justify & NOTLEFT ? 
+	       (labox->justify & RIGHT ? -tmpext.width :
+		-tmpext.width / 2) : 0);
+   points[2].x = points[3].x = points[0].x + tmpext.width;
+   points[0].y = points[3].y = (labox->justify & NOTBOTTOM ?
+	       (labox->justify & TOP ? -tmpext.ascent :
+		-(tmpext.ascent + tmpext.base) / 2) : -tmpext.base)
+		+ tmpext.descent;
+   points[1].y = points[2].y = points[0].y + tmpext.ascent - tmpext.descent;
+
+#ifdef SCHEMA
+   /* separate bounding box for pinlabels and infolabels */
+
+   if (labox->pin)
+      for (j = 0; j < 4; j++)
+	 pinadjust(labox->justify, &points[j].x, &points[j].y, 1);
+#endif
+
+   UTransformPoints(points, npoints, 4, labox->position,
+		labox->scale, labox->rotation); 
+}
+
+/*--------------------------------------------------------------*/
+/* Wrapper for single call to calcbboxsingle() in the netlister */
+/*--------------------------------------------------------------*/
+
+void calcinstbbox(genericptr *bboxgen, short *llx, short *lly, short *urx,
+		short *ury)
+{
+            
+   *llx = *lly = 32767;
+   *urx = *ury = -32768;
+
+   calcbboxsingle(bboxgen, llx, lly, urx, ury,
+#ifdef SCHEMA
+					       NULL, NULL, NULL, NULL
+#endif
+								     );
+}           
+
+/*----------------------------------------------------------------------*/
+/* Bounding box calculation for a single generic element		*/
+/*----------------------------------------------------------------------*/
 
 void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
 		short *ury
@@ -865,24 +943,8 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
    switch((*bboxgen)->type) {
 
       case(OBJECT):
+	 objinstbbox(TOOBJINST(bboxgen), npoints);
 
-#ifdef SCHEMA
-         points[0].x = points[1].x = TOOBJINST(bboxgen)->thisobject->lleft2.x;
-         points[1].y = points[2].y = TOOBJINST(bboxgen)->thisobject->lleft2.y
-		+ TOOBJINST(bboxgen)->thisobject->height2;
-         points[2].x = points[3].x = TOOBJINST(bboxgen)->thisobject->lleft2.x
-		+ TOOBJINST(bboxgen)->thisobject->width2;
-         points[0].y = points[3].y = TOOBJINST(bboxgen)->thisobject->lleft2.y;
-#else
-         points[0].x = points[1].x = TOOBJINST(bboxgen)->thisobject->lowerleft.x;
-         points[1].y = points[2].y = TOOBJINST(bboxgen)->thisobject->lowerleft.y
-		+ TOOBJINST(bboxgen)->thisobject->height;
-         points[2].x = points[3].x = TOOBJINST(bboxgen)->thisobject->lowerleft.x
-		+ TOOBJINST(bboxgen)->thisobject->width;
-         points[0].y = points[3].y = TOOBJINST(bboxgen)->thisobject->lowerleft.y;
-#endif
-         UTransformPoints(points, npoints, 4, TOOBJINST(bboxgen)->position,
-		 TOOBJINST(bboxgen)->scale, TOOBJINST(bboxgen)->rotation); 
          for (j = 0; j < 4; j++) {
             bboxcalc(npoints[j].x, llx, urx);
             bboxcalc(npoints[j].y, lly, ury);
@@ -891,29 +953,18 @@ void calcbboxsingle(genericptr *bboxgen, short *llx, short *lly, short *urx,
 
       case(LABEL):
 	 {
-	    TextExtents tmpext;
-            tmpext = ULength(TOLABEL(bboxgen)->string, NORMINST, 0.0, 0, NULL);
-            points[0].x = points[1].x = (TOLABEL(bboxgen)->justify & NOTLEFT ? 
-	       (TOLABEL(bboxgen)->justify & RIGHT ? -tmpext.width :
-		-tmpext.width / 2) : 0);
-            points[0].y = points[3].y = (TOLABEL(bboxgen)->justify & NOTBOTTOM ?
-	       (TOLABEL(bboxgen)->justify & TOP ? -tmpext.ascent :
-		-(tmpext.ascent + tmpext.base) / 2) : -tmpext.base);
-            points[1].y = points[2].y = points[0].y + tmpext.ascent - tmpext.descent;
-            points[2].x = points[3].x = points[0].x + tmpext.width;
+	    labelbbox(TOLABEL(bboxgen), npoints, NORMINST);
 
-            UTransformPoints(points, npoints, 4, TOLABEL(bboxgen)->position,
-		TOLABEL(bboxgen)->scale, TOLABEL(bboxgen)->rotation); 
 #ifdef SCHEMA
-	    /* separate bounding box for pinlabels and infolabels */
-
 	    if (TOLABEL(bboxgen)->pin) {
                for (j = 0; j < 4; j++) {
-		  pinadjust(TOLABEL(bboxgen)->justify, &npoints[j].x,
-			&npoints[j].y, 1);
                   bboxcalc(npoints[j].x, pllx, purx);
                   bboxcalc(npoints[j].y, plly, pury);
                }
+
+	       /* Include the pin's position in the bounding box */
+               bboxcalc(TOLABEL(bboxgen)->position.x, pllx, purx);
+               bboxcalc(TOLABEL(bboxgen)->position.y, plly, pury);
 	    }
 	    else
 #endif
