@@ -1326,6 +1326,10 @@ void newpagemenu(Widget w, u_int value, caddr_t nulldata)
       eventmode = NORMAL_MODE;
       catreturn();
    }
+   else if (eventmode != NORMAL_MODE && eventmode != COPY2_MODE) {
+      Wprintf("Cannot switch pages from this mode");
+      return;
+   }
    newpage((short)value);
 }
 
