@@ -37,6 +37,7 @@
 
 extern Display  *dpy;
 extern int *appcolors;
+extern float version;
 
 /*----------------------------------------------------------------------*/
 #ifdef SCHEMA
@@ -79,8 +80,12 @@ int devindex;   /* count for labeling devices in the flattened netlist */		  int
 void createnets(Boolean keepcalls, char *mode)
 {
    if (setobjecttype(objectdata, mode)) {
+
+      Wprintf("Generating netlists");
+
       gennetlist(objectdata);
       gencalllist(objectdata, keepcalls);
+      Wprintf("Finished netlists");
    }
    else
       Wprintf("Error:  attempt to generate netlist for a symbol.");
@@ -309,9 +314,10 @@ void gennetlist(objectptr cschem)
 /* the sibling object.						*/
 /*--------------------------------------------------------------*/
 
-void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
+void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop,
+	short llx, short lly, short urx, short ury)
 {
-   XPoint *tmppts;
+   XPoint *tmppts, sbbox[2];
    int i, j;
    labelptr olabel;
    polyptr tpoly;
@@ -343,8 +349,8 @@ void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
 	       /* Transform all the way up to the level above cinst */
 	       for (psearch = schemtop; psearch != NULL; psearch = psearch->nextpair) {
 		  subsibinst = psearch->thisinst;
-	          UTransformPoints(tmppts, tmppts, 1,
-			subsibinst->position, subsibinst->scale, subsibinst->rotation);
+	          UTransformPoints(tmppts, tmppts, 1, subsibinst->position,
+				subsibinst->scale, subsibinst->rotation);
 	       }
 	       searchconnect(tmppts, 1, cinst);
 	    }
@@ -373,8 +379,8 @@ void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
 	       /* Transform all the way up to the level above cinst */
 	       for (psearch = schemtop; psearch != NULL; psearch = psearch->nextpair) {
 		  subsibinst = psearch->thisinst;
-	          UTransformPoints(tmppts, tmppts, 1,
-			subsibinst->position, subsibinst->scale, subsibinst->rotation);
+	          UTransformPoints(tmppts, tmppts, 1, subsibinst->position,
+			subsibinst->scale, subsibinst->rotation);
 	       }
 	       searchconnect(tmppts, 1, cinst);
 	    }
@@ -392,8 +398,8 @@ void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
 	       /* Transform all the way up to the level above cinst */
 	       for (psearch = schemtop; psearch != NULL; psearch = psearch->nextpair) {
 		  subsibinst = psearch->thisinst;
-	          UTransformPoints(tmppts, tmppts, 1,
-			subsibinst->position, subsibinst->scale, subsibinst->rotation);
+	          UTransformPoints(tmppts, tmppts, 1, subsibinst->position,
+				subsibinst->scale, subsibinst->rotation);
 	       }
 	       searchconnect(tmppts, tpoly->number, cinst);
 	    }
@@ -405,6 +411,20 @@ void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
       for (i = 0; i < sibling->parts; i++) {
 	 iseek = sibling->plist + i;
 	 if ((*iseek)->type == OBJECT) {
+
+	    /* Don't search this instance unless the bounding box 	*/
+	    /* overlaps the bounding box of the calling object.		*/
+
+	    calcinstbbox(iseek, &sbbox[0].x, &sbbox[0].y, &sbbox[1].x, &sbbox[1].y);
+	    for (psearch = schemtop; psearch != NULL; psearch = psearch->nextpair) {
+	       subsibinst = psearch->thisinst;
+	       UTransformPoints(sbbox, sbbox, 2, subsibinst->position,
+			subsibinst->scale, subsibinst->rotation);
+	    }
+	    if ((llx > sbbox[1].x) || (urx < sbbox[0].x) || (lly > sbbox[1].y)
+			|| (ury < sbbox[0].y))
+	       continue;
+
 	    subsibinst = TOOBJINST(iseek);
 
 	    /* push stack */
@@ -414,7 +434,7 @@ void search_on_siblings(objinstptr cinst, objinstptr isib, objectpair *schemtop)
 	    newpair->nextpair = schemtop;
 	    schemtop = newpair;
 
-	    search_on_siblings(cinst, subsibinst, schemtop);
+	    search_on_siblings(cinst, subsibinst, schemtop, llx, lly, urx, ury);
 
 	    /* pop stack */
 
@@ -447,6 +467,7 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
    objinstptr cinst, isib;
    objectptr callobj, callsymb, sibling;
    XPoint xpos;
+   short ibllx, iblly, iburx, ibury, sbllx, sblly, sburx, sbury;
    int i, j, netfrom, netto;
    labelptr olabel;
    polyptr tpoly;
@@ -454,6 +475,8 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
    PolylistPtr pseek;
    LabellistPtr lseek;
    
+   cschem->traversed = True;	/* This object has been dealt with */
+
    for (i = 0; i < cschem->parts; i++) {
       cgen = cschem->plist + i;
       if ((*cgen)->type == OBJECT) {
@@ -472,13 +495,17 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
 
 	 callsymb = cinst->thisobject;
 
+	 /* Note:  callobj is the next schematic in the hierarchy.	*/
+	 /* callsymb is the next visible object in the hierarchy, 	*/
+	 /* which may be either a schematic or a symbol.		*/
+
 	 /*-------------------------------------------------------------*/
 	 /* For object instances which are their own schematics (i.e.,	*/
 	 /* have netlist elements), don't rely on any pin list but make	*/
 	 /* a survey of how polygons connect into the object.		*/
 	 /*-------------------------------------------------------------*/
 
-	 if (cinst->thisobject->symschem == NULL
+	 if (callsymb->symschem == NULL
 		&& callobj->schemtype != FUNDAMENTAL
 		&& callobj->schemtype != TRIVIAL) {
 
@@ -509,13 +536,24 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
 	    /* the object under consideration.  Search for connections	*/
 	    /* between the two objects.					*/
 
+	    calcinstbbox(cgen, &ibllx, &iblly, &iburx, &ibury);
+
 	    /* Only need to look forward from the current position. */
    	    for (j = i + 1; j < cschem->parts; j++) {
 
 	       iseek = cschem->plist + j;
 	       if ((*iseek)->type == OBJECT) {
-		  isib = TOOBJINST(iseek);
-		  search_on_siblings(cinst, isib, NULL);
+		  calcinstbbox(iseek, &sbllx, &sblly, &sburx, &sbury);
+
+		  /* Check intersection of the two object instances;	*/
+		  /* don't do a search if they are disjoint.		*/
+
+		  if ((ibllx <= sburx) && (iburx >= sbllx) &&
+				(iblly <= sbury) && (ibury >= sblly)) {
+		     isib = TOOBJINST(iseek);
+		     search_on_siblings(cinst, isib, NULL,	
+				ibllx, iblly, iburx, ibury);
+		  }
 	       }
 	    }
 	 }
@@ -524,7 +562,7 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
 	 /* Recursively call gencalllist() on the schematic.		*/
 	 /*-------------------------------------------------------------*/
 
-	 if (callobj->calllist == NULL)
+	 if (callobj->traversed == False)
 	    gencalllist(callobj, keepcalls);
 
 	 /*-------------------------------------------------------------*/
@@ -564,9 +602,14 @@ void gencalllist(objectptr cschem, Boolean keepcalls)
 		  /* Pin in symbol doen't have a corresponding pin in the */
 		  /* schematic.  This is not necessarily an error.	  */
 
-		  if (netto == 0)
+		  if (netto == 0) {
+		     char *snew = textprint(olabel->string, NULL);
 		     fprintf(stderr,
-			"Warning:  pin in symbol does not exist in schematic");
+			"Warning:  pin \"%s\" in symbol \"%s\" does"
+			" not connect to anything in schematic \"%s\"\n",
+			snew, callsymb->name, callobj->name);
+		     free(snew);
+		  }
 
 		  /* Pass global net up the hierarchy without making a call. */
 		  else if (netto < 0)
@@ -1000,10 +1043,9 @@ Boolean searchconnect(XPoint *points, int number, objinstptr cinst)
 	 cgen = cobj->plist + i;
          if ((*cgen)->type != OBJECT) continue;
 	 tinst = TOOBJINST(cgen);
-	 if (tinst->thisobject->symschem != NULL)
-	    tobj = tinst->thisobject->symschem;
-	 else
-            tobj = tinst->thisobject;
+
+	 /* Look at the object only (symbol, or schematic if it has no symbol) */
+         tobj = tinst->thisobject;
 
 	 /* Search for connections to pin labels */
 
@@ -1026,9 +1068,15 @@ Boolean searchconnect(XPoint *points, int number, objinstptr cinst)
 	    }
 	 }
       }
-      for (nseek = cobj->netlist; nseek != NULL; nseek = nseek->next) {
-	 for (pseek = nseek->polygons; pseek != NULL; pseek = pseek->next) {
-	    tpoly = pseek->poly;
+
+      /* mergenets() may alter the netlist, so search on polygons in the */
+      /* parts list, not polygons in the netlist.			 */
+
+      for (i = 0; i < cobj->parts; i++) {
+	 ogen = cobj->plist + i;
+	 if ((*ogen)->type == POLYGON) {
+	    tpoly = TOPOLY(ogen);
+	    if (nonnetwork(tpoly)) continue;
 
 	    /* Search for connections from polygons to polygons */
 
@@ -1129,7 +1177,7 @@ int onsegment(XPoint *af, XPoint *at, XPoint *bf, XPoint *bt, objectptr cschem)
 /*	       newnet - new id to be changed to				*/	
 /*----------------------------------------------------------------------*/
 
-void mergenets(objectptr cschem, int orignet, int newnet) 
+NetlistPtr mergenets(objectptr cschem, int orignet, int newnet) 
 {
    NetlistPtr origlist, newlist, linkpoint;
    PolylistPtr plist;
@@ -1153,7 +1201,7 @@ void mergenets(objectptr cschem, int orignet, int newnet)
    }
    else if ((origlist == NULL) || (newlist == NULL)) {
       fprintf(stderr, "Bad net number passed to mergenets()!\n");
-      return;
+      return NULL;
    }
 
    if (newlist != NULL) {
@@ -1197,6 +1245,8 @@ void mergenets(objectptr cschem, int orignet, int newnet)
       for (portlist = calllist->ports; portlist != NULL; portlist = portlist->next)
          if (portlist->netid == orignet)
 	    portlist->netid = newnet;
+
+   return linkpoint;
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1601,7 +1651,7 @@ int pintonet(objectptr cschem, labelptr testpin)
       for (netlist = cschem->netlist; netlist != NULL; netlist = netlist->next)
          for (seeklabel = netlist->labels; seeklabel != NULL;
 		seeklabel = seeklabel->next)
-            if (!stringcomp(seeklabel->label->string, testpin->string))
+            if (!stringcomprelaxed(seeklabel->label->string, testpin->string))
 	       return (netlist->netid);
    }
 
@@ -1611,7 +1661,7 @@ int pintonet(objectptr cschem, labelptr testpin)
       for (netlist = globallist; netlist != NULL; netlist = netlist->next)
          for (seeklabel = netlist->labels; seeklabel != NULL;
 		seeklabel = seeklabel->next)
-            if (!stringcomp(seeklabel->label->string, testpin->string))
+            if (!stringcomprelaxed(seeklabel->label->string, testpin->string))
 	       return (netlist->netid);
    }
    return 0;
@@ -1935,7 +1985,8 @@ void writeglobals(objectptr cschem, FILE *fp)
    for (gptr = globallist; gptr != NULL; gptr = gptr->next) {
       gpin = gptr->labels->label;	/* Only print first label in list (?) */
       snew = textprint(gpin->string, NULL);
-      fprintf(fp, "%s = %d\n", snew, -gptr->netid);
+      fprintf(fp, ".GLOBAL %s\n", snew);		/* hspice format */
+      /* fprintf(fp, "%s = %d\n", snew, -gptr->netid); */ /* generic format */
       free(snew);
    }
    fprintf(fp, "\n");
@@ -1949,8 +2000,9 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
 {
    CalllistPtr calllist = cschem->calllist;
    PortlistPtr portlist, plist;
-   int netid, pnet, portid;
+   int netid, pnet, portid, length, plen;
    char *stsave, *pstring;
+   NetlistPtr gptr;
 
    /* make sure that all the subcircuits have been written first */
 
@@ -1970,6 +2022,7 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
 
    else if (portlist != NULL) {
       fprintf(fp, ".subckt %s", cschem->name);
+      length = 9 + strlen(cschem->name);
 
       /* List of parameters in subcircuit. 			  */
       /* Each parameter connects to a net which may have multiple */
@@ -1980,6 +2033,12 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
       for (; portlist != NULL; portlist = portlist->next) {
          netid = portlist->netid;
 	 pstring = textprint(nettopin(netid, cschem, NULL), NULL);
+	 plen = strlen(pstring) + 1;
+	 if (length + plen > 78) {
+            fprintf(fp, " \\\n");	/* Line break */
+	    length = 0;
+	 }
+	 else length += plen;
          fprintf(fp, " %s", pstring);
 	 free(pstring);
       }
@@ -1996,6 +2055,7 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
 
          fprintf(fp, "X%d", subindex++);
 	 stsave = calllist->callobj->name;
+         length = 6;
 
 	 /* The object's definition lists calls in the order of the object's	*/
 	 /* port list.  Therefore, use this order to find the port list.  	*/
@@ -2011,9 +2071,17 @@ void writehierarchy(objectptr cschem, CalllistPtr cfrom, FILE *fp)
 
 	    pnet = (plist == NULL) ?  netmax(cschem) + 1 : plist->netid;
 	    pstring = textprint(nettopin(pnet, cschem, NULL), NULL);
+	    plen = strlen(pstring) + 1;
+	    if (length + plen > 78) {
+               fprintf(fp, " \\\n");	/* Line break */
+	       length = 0;
+	    }
+	    else length += plen;
 	    fprintf(fp, " %s", pstring);
 	    free(pstring);
          }
+	 plen = 1 + strlen(stsave);
+	 if (length + plen > 78) fprintf(fp, " \\\n");	/* Line break */
          fprintf(fp, " %s\n", stsave);
       }
    }
@@ -2031,6 +2099,8 @@ void writenet(objectptr cschem, char *mode, char *suffix)
    char *prefix, *cpos;
    FILE *fp;
 
+   prefix = (char *)malloc(sizeof(char));
+   *prefix = '\0';
    devindex = subindex = 1;
  
    if ((cpos = strchr(cschem->name, ':')) != NULL) *cpos = '\0';
@@ -2039,22 +2109,22 @@ void writenet(objectptr cschem, char *mode, char *suffix)
 
    if ((fp = fopen(filename, "w")) != NULL) {
       if (!strcmp(mode, "spice")) {
-         /* writeglobals(cschem, fp); */
-         fprintf(fp, "SPICE circuit %s\n\n", cschem->name);
+         fprintf(fp, "*SPICE circuit \"%s\" from XCircuit v%3.2f\n\n",
+		cschem->name, version);
+	 cleartraversed(cschem);
+         writeglobals(cschem, fp);
          writehierarchy(cschem, NULL, fp);
       }
-      else {
-         prefix = (char *)malloc(sizeof(char));
-	 *prefix = '\0';
-         if (!strcmp(mode, "sim"))
-	    writesimflat(cschem, NULL, prefix, fp);
-         else if (!strcmp(mode, "pcb")) {
-	    ptable = (struct Ptab *)NULL;
-	    writepcb(cschem, NULL, "");
-	    outputpcb(cschem, fp);
-	    freepcb();
-	 }
-         free(prefix);
+      else if (!strcmp(mode, "sim")) {
+         fprintf(fp, "| sim circuit \"%s\" from XCircuit v%3.2f\n",
+		cschem->name, version);
+	 writesimflat(cschem, NULL, prefix, fp);
+      }
+      else if (!strcmp(mode, "pcb")) {
+	 ptable = (struct Ptab *)NULL;
+	 writepcb(cschem, NULL, "");
+	 outputpcb(cschem, fp);
+	 freepcb();
       }
       fclose(fp);
       sprintf(_STR, "%s netlist saved as %s", mode, filename);
@@ -2063,6 +2133,7 @@ void writenet(objectptr cschem, char *mode, char *suffix)
       sprintf(_STR, "Could not open file %s for writing.", filename);
    }
    Wprintf(_STR);
+   free(prefix);
 }
 
 /*----------------------------------------------------------------------*/
@@ -2375,6 +2446,39 @@ void freenetlist(objectptr cschem)
       cschem->netlist = NULL;
 }
 
+/*----------------------------------------------------------------------*/
+/* Clear the "traversed" flag in all objects of the hierarchy.		*/
+/*----------------------------------------------------------------------*/
+
+void cleartraversed(objectptr cschem)
+{
+   CalllistPtr calllist, cptr;
+   PortlistPtr portlist, pptr;
+   genericptr *cgen;
+   objinstptr cinst;
+   objectptr callobj;
+	
+   /* Recursively call cleartraversed() on all subobjects, a la gennetlist()  */
+   /* Use the parts list of the object, not the calllist, because calls */
+   /* may have been removed.						*/
+
+   for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
+      if ((*cgen)->type == OBJECT) {
+ 	 cinst = TOOBJINST(cgen);
+
+	 if (cinst->thisobject->symschem != NULL)
+	    callobj = cinst->thisobject->symschem;
+	 else
+	    callobj = cinst->thisobject;
+	
+	 /* Don't infinitely recurse if object is on its own schematic */
+
+	 if (callobj != cschem) cleartraversed(callobj);
+      }
+   }
+   cschem->traversed = False;
+}
+
 /*-------------------------------------------------------------------------*/
 /* Free memory allocated for netlists, portlists, and calllists		   */
 /*-------------------------------------------------------------------------*/
