@@ -39,6 +39,56 @@ extern objectpair *pushlist;
 extern short textpos, textend;
 extern char _STR[150];
 
+/*------------------------------------------------------*/
+/* If the instance comes from the library, replace the	*/
+/* default value with the instance value.		*/
+/*------------------------------------------------------*/
+
+void replaceparams(objinstptr thisinst)
+{
+   objectptr thisobj;
+   oparamptr ops, ips;
+   int i, nullparms = 0;
+
+   thisobj = thisinst->thisobject;
+
+   for (i = 0; i < thisobj->num_params; i++) {
+      ops = thisobj->params[i];
+      ips = thisinst->params[i];
+      if (ips == NULL) {
+	 nullparms++;
+	 continue;  /* this parameter is already default */
+      }
+
+      switch(ops->type) {
+	 case XC_STRING:
+	    if (stringcomp(ops->parameter.string, ips->parameter.string)) {
+	       freelabel(ops->parameter.string);
+	       ops->parameter.string = ips->parameter.string;
+	       free(ips);
+	       thisinst->params[i] = NULL;
+	       nullparms++;
+	    }
+	    break;
+	 case XC_SHORT:
+	    if (ops->parameter.svalue != ips->parameter.svalue) {
+	       free(ops);
+	       ops = ips;
+	       thisinst->params[i] = NULL;
+	       nullparms++;
+	    }
+	    break;
+      }
+   }
+
+   /* Object instance takes all default values---remove the params list */
+
+   if (nullparms == thisobj->num_params) {
+      free(thisinst->params);
+      thisinst->params = NULL;
+   }
+}
+
 /*------------------------------------------------------*/
 /* Resolve differences between the object instance	*/
 /* parameters and the default parameters.  If they	*/
@@ -55,6 +105,12 @@ void resolveparams(objinstptr thisinst)
    /* If the instance has no parameters itself, ignore it. */
    if (thisinst == NULL || thisinst->params == NULL) return;
 
+   if (checklibtop() != NULL) {
+      /* printf("Came from library or top page:  changing default value\n"); */
+      replaceparams(thisinst);
+      return;
+   }
+
    thisobj = thisinst->thisobject;
 
    for (i = 0; i < thisobj->num_params; i++) {
@@ -68,6 +124,7 @@ void resolveparams(objinstptr thisinst)
       switch(ops->type) {
 	 case XC_STRING:
 	    if (!stringcomp(ops->parameter.string, ips->parameter.string)) {
+	       freelabel(ips->parameter.string);
 	       free(ips);
 	       thisinst->params[i] = NULL;
 	       nullparms++;
@@ -499,10 +556,12 @@ void unparameterize(u_int mode)
 
    if ((areastruct.selects == 1) && (ptype & LABEL) && textend > 0
 		&& textend < textpos) {
+      if (SELECTTYPE(areastruct.selectlist) != LABEL) return;	 /* Not a label */
       settext = SELTOLABEL(areastruct.selectlist);
       strptr = findstringpart(textend, &locpos, settext->string, NORMINST);
       while (strptr != NULL && strptr->type != PARAM_END)
 	 strptr = strptr->nextpart;
+      if (strptr == NULL) return;	/* No parameters */
       tmpptr = settext->string;
       while (tmpptr != NULL && tmpptr->type != PARAM_START &&
 	 	tmpptr->nextpart != strptr->nextpart)
@@ -583,10 +642,7 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
 objectptr checklibtop()
 {
    objectptr pobj;
-   objectpair *thispush;
-
-   if (pushlist == NULL) return NULL;
-   else thispush = pushlist->nextpair;
+   objectpair *thispush = pushlist;
 
    while (thispush != NULL) {
       pobj = thispush->thisobject;
