@@ -158,7 +158,7 @@ extern void loadlibrary(short);
 extern void startloadfile();
 extern void importfile();
 extern void loadfile(short);
-extern void readlabel(objectptr, char *, stringpart **, short *, float *, float *);
+extern void readlabel(objectptr, char *, stringpart **);
 extern u_char *find_match(u_char *);
 extern void varscan(objectptr, char *, short *);
 extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int);
@@ -170,7 +170,7 @@ extern void findfonts(objectptr, short *);
 extern void savefile(objectptr, short);
 extern int printRGBvalues(char *, int, const char *);
 extern int nosprint(char *);
-extern short writelabel(FILE *, stringpart *, short *, float);
+extern short writelabel(FILE *, stringpart *, short *);
 extern void writesegment(stringpart *, float *, int *);
 extern void writelabelsegs(FILE *, short *, stringpart *);
 extern void printobjects(FILE *, objectptr, float, float, float, short, short,
@@ -227,6 +227,9 @@ extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
 extern void manhattanize(XPoint *, polyptr);
 extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
+extern void objinstbbox(objinstptr, XPoint *);
+extern void labelbbox(labelptr, XPoint *, objinstptr);
+extern void calcinstbbox(genericptr *, short *, short *, short *, short *);
 extern void calcbboxsingle(genericptr *, short *, short *, short *, short *
 #ifdef SCHEMA
                            ,short *, short *, short *, short *
@@ -407,7 +410,8 @@ extern Boolean nonnetwork(polyptr);
 extern int globalmax();
 extern void gennetlist(objectptr);
 extern void gencalllist(objectptr, Boolean);
-extern void search_on_siblings(objinstptr, objinstptr, objectpair *);
+extern void search_on_siblings(objinstptr, objinstptr, objectpair *,
+		short, short, short, short);
 extern int is_resolved(genericptr *, objectpair *, objectptr *);
 extern void highlightnet(objectptr, objinstptr, int);
 extern int pushnetwork(objectpair *, objectptr);
@@ -418,7 +422,7 @@ extern Boolean searchconnect(XPoint *, int, objinstptr);
 extern void addpoly(objectptr, polyptr, int);
 extern long zsign(long, long);
 extern int onsegment(XPoint *, XPoint *, XPoint *, XPoint *, objectptr);
-extern void mergenets(objectptr, int, int);
+extern NetlistPtr mergenets(objectptr, int, int);
 extern void removecall(objectptr, CalllistPtr);
 extern void addpin(objectptr, labelptr, int);
 extern void addglobal(labelptr, int);
@@ -441,6 +445,7 @@ extern void outputpcb(objectptr, FILE *);
 extern void freepcb();
 extern void freenets(objectptr);
 extern void freeglobals();
+extern void cleartraversed(objectptr);
 extern void clearpins(objectptr);
 #endif
 
@@ -516,7 +521,7 @@ extern void findsubschems(int, objectptr);
 extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
-extern labelptr findotherpin(labelptr);
+extern void changeotherpins(labelptr, stringpart *);
 extern void swapschem(Widget, u_int, caddr_t);
 extern void schemdisassoc();
 extern void startschemassoc(Widget, caddr_t, caddr_t);
@@ -560,6 +565,7 @@ extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
+extern int stringcomprelaxed(stringpart *, stringpart *);
 extern int stringparts(stringpart *);
 extern int stringlength(stringpart *, Boolean, objinstptr);
 extern stringpart *stringcopy(stringpart *);
