@@ -1,7 +1,10 @@
 /*-------------------------------------------------------------------------*/
 /* rcfile.c --- Load initialization variables from the .xcircuitrc file	   */
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
+
+/* This file is made obsolete by the Python interpreter, if compiled in */
+#ifndef HAVE_PYTHON
    
 /*-------------------------------------------------------------------------*/
 /*      written by Tim Edwards, 9/9/96                                     */
@@ -10,7 +13,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -23,6 +28,11 @@
 #include "xcircuit.h"
 #include "menudep.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*----------------------------------------------------------------------*/
 /* External variable declarations					*/
 /*----------------------------------------------------------------------*/
@@ -39,15 +49,11 @@ extern Window win;
 
 /*----------------------------------------------------------------------*/
 
-extern void setdefaultfontmarks();
-void defaultscript();
-
-/*----------------------------------------------------------------------*/
-
 #define LIBOVERRIDE	1
 #define LIBLOADED	2
 #define COLOROVERRIDE	4
 #define FONTOVERRIDE	8
+#define KEYOVERRIDE	16
 
 /*----------------------------------------------------------------------*/
 /* Execute a single command from a script or from the command line	*/
@@ -56,7 +62,6 @@ void defaultscript();
 short execcommand(short pflags, char *cmdptr)
 {
    short flags = pflags;
-   short i;
    Boolean a;
    char type[50], value[50];
    char *argptr;
@@ -129,6 +134,9 @@ short execcommand(short pflags, char *cmdptr)
       else if (!strcmp(value, "font") || !strcmp(value, "fonts")) {
 	 flags |= FONTOVERRIDE;
       }
+      else if (!strcmp(value, "key") || !strcmp(value, "keybindings")) {
+	 flags |= KEYOVERRIDE;
+      }
    }
 
    /* load builtin libraries */
@@ -176,6 +184,35 @@ short execcommand(short pflags, char *cmdptr)
       addnewcolorentry(xc_alloccolor(_STR2));
    }
 
+   /* create macro */
+
+   else if (!strcmp(cmdptr, "bind")) {
+      int args;
+      if ((args = sscanf(argptr, "%s %s", _STR, _STR2)) == 1) {
+	 char *binding;
+	 int func = -1, keywstate;
+	 keywstate = string_to_key(_STR);
+	 if (keywstate == 0) { /* Perhaps 1st arg is a function? */
+	    keywstate = -1;
+	    func = string_to_func(_STR, NULL);
+	 }
+	 binding = binding_to_string(keywstate, func);
+	 Wprintf(binding);
+	 free(binding);
+      }
+      else if (args == 0)
+	 Wprintf("See help window for list of bindings");
+      else
+         add_keybinding(_STR, _STR2);
+   }
+
+   /* destroy macro */
+
+   else if (!strcmp(cmdptr, "unbind")) {
+      sscanf(argptr, "%s %s", _STR, _STR2);
+      remove_keybinding(_STR, _STR2);
+   }
+
    /* beep mode */
 
    else if (!strcmp(cmdptr, "beep")) {
@@ -185,7 +222,7 @@ short execcommand(short pflags, char *cmdptr)
 
    /* active delay */
 
-   else if (!strcmp(cmdptr, "delay")) {
+   else if (!strcmp(cmdptr, "pause")) {
       float dval = 0;
       sscanf(argptr, "%f", &dval);
       usleep((int)(1e6 * dval));
@@ -199,7 +236,7 @@ short execcommand(short pflags, char *cmdptr)
       refresh(NULL, NULL, NULL);
 
       while(XCheckWindowEvent(dpy, win, ~NoEventMask, &event))
-	 XtDispatchEvent(&event);
+         XtDispatchEvent(&event);
    }
 
    /* set variables */
@@ -256,7 +293,7 @@ short execcommand(short pflags, char *cmdptr)
 	    xobjs.pagelist[cpage]->coordstyle = CM;
 	    xobjs.pagelist[cpage]->pagesize.x = 595;  /* A4 size */
 	    xobjs.pagelist[cpage]->pagesize.y = 842;
-	    togglegrid((unsigned short)xobjs.pagelist[cpage]->coordstyle);
+	    togglegrid((u_short)xobjs.pagelist[cpage]->coordstyle);
          }
       }
       else if (!strncmp(argptr, "orient", 6)) {   /* "orient" or "orientation" */
@@ -296,8 +333,7 @@ short readcommand(short mode, FILE *fd)
 {
    char *temp, *tmpptr, *endptr;
    short templen = TEMPLEN, tempmax = TEMPLEN;
-   short flags = (mode == 0) ? 0 : LIBOVERRIDE | LIBLOADED |
-		COLOROVERRIDE | FONTOVERRIDE;
+   short flags = (mode == 0) ? 0 : LIBOVERRIDE | LIBLOADED | FONTOVERRIDE;
 
    temp = (char *)malloc(templen);
    tmpptr = temp;
@@ -366,7 +402,7 @@ void execscript()
 /* Execute the .xcircuitrc startup script				*/
 /*----------------------------------------------------------------------*/
 
-loadrcfile()
+void loadrcfile()
 {
    char *userdir = getenv((const char *)"HOME");
    FILE *fd;
@@ -414,6 +450,13 @@ loadrcfile()
       addnewcolorentry(xc_alloccolor("Tan"));
       addnewcolorentry(xc_alloccolor("Brown"));
    }
+
+   /* Add the default key bindings */
+
+   if (!(flags & KEYOVERRIDE))
+      default_keybindings();
 }
 
+#endif
+/* #endif for ifndef HAVE_PYTHON */
 /*-------------------------------------------------------------------------*/
