@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------*/
 /* render.c --- Ghostscript rendering of background PostScript files	*/
-/* Copyright (c) 2000  Tim Edwards, Johns Hopkins University        	*/
+/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
@@ -10,7 +10,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if !defined(__DARWIN__)
 #include <malloc.h>
+#endif
 #include <time.h>
 #include <sys/types.h>
 #include <signal.h>
@@ -26,6 +28,11 @@
 #include "colordefs.h"
 #include "xcircuit.h"
 
+/*----------------------------------------------------------------------*/
+/* Function prototype declarations                                      */
+/*----------------------------------------------------------------------*/
+#include "prototypes.h"
+
 /*------------------------------------------------------------------------*/
 /* External Variable definitions                                          */
 /*------------------------------------------------------------------------*/
@@ -36,7 +43,6 @@ extern Clientdata areastruct;
 extern Display *dpy;
 extern Window win;
 extern short eventmode;
-extern short pushes;
 extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
@@ -51,14 +57,6 @@ extern Pixmap dbuf;
 #define vfork fork
 #endif
 
-/*------------------------------*/
-/* Declarations of functions 	*/
-/*------------------------------*/
-
-void loadbackground();
-void call_loadbg(char *);
-extern float getpsscale(float, short);
-
 /*------------------------------------------------------*/
 /* Global variable definitions				*/
 /*------------------------------------------------------*/
@@ -191,7 +189,7 @@ void wait_for_done()
 void start_gs()
 {
    int std_out[2], std_err[2], ret;
-#ifdef SYSV
+#ifdef SVR4
    static char env_str1[128], env_str2[64];
 #endif
 
@@ -229,7 +227,7 @@ void start_gs()
 	 close(std_err[1]);
 #endif
 
-#ifdef HAVE_PUTENV
+#ifdef SVR4
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
 #ifdef DOUBLEBUFFER
@@ -394,8 +392,8 @@ void backgroundbbox(int mpage)
 void readbackground(FILE *fi)
 {
    FILE *fbg = (FILE *)NULL;
-   char file_in[100] = "@/tmp/XXXXXX";  /* "@" denotes a temporary file */
    int tfd;
+   char file_in[20] = "@/tmp/XXXXXX";  /* "@" denotes a temporary file */
 
    tfd = mkstemp(file_in + 1);
    if (tfd == -1) printf("Error generating temporary filename\n");
@@ -404,7 +402,7 @@ void readbackground(FILE *fi)
 	 printf("Error opening temporary file \"%s\"\n", file_in + 1);
       }
    }
-      
+
    /* Read the file to the restore directive or end_insertion */
    /* Skip restore directive and end_insertion command */
 
@@ -426,7 +424,6 @@ void savebackground(FILE *fo, char *psfilename)
    FILE *psf;
    char *fname = psfilename;
    char line_in[256];
-   int psxpos, psypos;
 
    if (fname[0] == '@') fname++;
 
@@ -489,7 +486,6 @@ void send_to_gs(char *text)
 
 int renderbackground()
 {
-   int i;
    char *bgfile;
    float psnorm, psxpos, psypos, defscale;
    float devres = 0.96; /* = 72.0 / 75.0, ps_units/in : screen_dpi */
