@@ -564,26 +564,28 @@ int checksym(objectptr symobj, char *cname)
    return 0;
 }
 
-/*--------------------------------------------------------*/
-/* Find location of corresponding pin in symbol/schematic */
-/*--------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Find location of corresponding pin in symbol/schematic and change it */
+/* to the text of the indicated label.					*/
+/*----------------------------------------------------------------------*/
 
-labelptr findotherpin(labelptr curlabel)
+void changeotherpins(labelptr newlabel, stringpart *oldstring)
 {
    objectptr other = objectdata->symschem;
    genericptr *tgen;
    labelptr tlab;
 
-   if (areastruct.schemon == False || other == NULL || curlabel->pin == False)
-      return NULL;
+   if (other == NULL) return;
 
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
       if ((*tgen)->type == LABEL) {
 	 tlab = TOLABEL(tgen);
-	 if (!stringcomp(tlab->string, curlabel->string)) return tlab;
+	 if (!stringcomp(tlab->string, oldstring)) {
+	    free(tlab->string);
+	    tlab->string = stringcopy(newlabel->string);
+	 }
       }
    }
-   return NULL;
 }
 
 /*----------------------------------------------------------------------*/
