@@ -354,26 +354,8 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
 
       if ((*curgen)->type == (class & OBJECT)) {
 
-#ifdef SCHEMA
-#define lowerleft lleft2
-#define width width2
-#define height height2
-#endif
+	 objinstbbox(TOOBJINST(curgen), newboxpts);
 
-         bboxpts[0].x = bboxpts[1].x = TOOBJINST(curgen)->thisobject->lowerleft.x;
-         bboxpts[2].x = bboxpts[3].x = TOOBJINST(curgen)->thisobject->lowerleft.x + 
-	    TOOBJINST(curgen)->thisobject->width;
-         bboxpts[0].y = bboxpts[3].y = TOOBJINST(curgen)->thisobject->lowerleft.y;
-         bboxpts[1].y = bboxpts[2].y = TOOBJINST(curgen)->thisobject->lowerleft.y + 
-	    TOOBJINST(curgen)->thisobject->height;
-         UTransformPoints(bboxpts, newboxpts, 4, TOOBJINST(curgen)->position,
-		TOOBJINST(curgen)->scale, TOOBJINST(curgen)->rotation);
-
-#ifdef SCHEMA
-#undef lowerleft
-#undef width
-#undef height
-#endif
          /* Look for an intersect of the boundingbox and pointer position. */
          /* This is a simple matter of rotating the pointer position with  */
          /*  respect to the origin of the bounding box segment, as if the  */
@@ -388,32 +370,9 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
       else if ((*curgen)->type == (class & LABEL)) {
 	 labelptr curlab = TOLABEL(curgen);
 
-         /* now look among the labels */
+         /* Now look among the labels */
 
-	 TextExtents tmpext = ULength(curlab->string, selinst, 0.0, 0, NULL);
-
-         bboxpts[0].x = bboxpts[1].x = (curlab->justify & NOTLEFT ?
-	    (curlab->justify & RIGHT ? -tmpext.width : -tmpext.width / 2) : 0);
-	 bboxpts[2].x = bboxpts[3].x = bboxpts[0].x + tmpext.width;
-         bboxpts[0].y = bboxpts[3].y = (curlab->justify & NOTBOTTOM ?
-	    (curlab->justify & TOP ? -tmpext.ascent :
-	    -(tmpext.ascent + tmpext.base) / 2) : -tmpext.base)
-	    + tmpext.descent;
-	 bboxpts[1].y = bboxpts[2].y = bboxpts[0].y +
-		(tmpext.ascent - tmpext.descent);
-
-#ifdef SCHEMA
-	 if (curlab->pin) {
-	    int i;
-
-	    if (!areastruct.schemon) continue;
-	    for (i = 0; i < 4; i++) 
-	       pinadjust(curlab->justify, &(bboxpts[i].x), &(bboxpts[i].y), 1);
-	 }
-#endif
-
-         UTransformPoints(bboxpts, newboxpts, 4, curlab->position,
-		curlab->scale, curlab->rotation);
+	 labelbbox(curlab, newboxpts, selinst);
 
          /* check for point inside bounding box, as for objects */
 
