@@ -63,7 +63,7 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
    stringpart *newptr, *lastptr, *nextptr;
 
    newptr = (stringpart *)malloc(sizeof(stringpart));
-   newptr->data = NULL;
+   newptr->data.string = NULL;
 
    if (before == *strhead) {	/* insert at beginning */
       newptr->nextpart = *strhead;
@@ -192,6 +192,7 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
    }
    if (dstr->type == TEXT_STRING) free(dstr->data.string);
    free(dstr);
+   return strptr;
 }
 
 /*----------------------------------------------------------------------*/
@@ -495,9 +496,73 @@ int stringcomp(stringpart *string1, stringpart *string2)
 	 }
       }
    }
+
+   /* One string continues after the other ends. . . */ 
+   if (strptr1 != NULL || strptr2 != NULL) return 1;
    return 0;
 }
 
+/*----------------------------------------------------------------------*/
+/* Test if the specified font is in the "Symbol" font family.		*/
+/*----------------------------------------------------------------------*/
+
+Boolean issymbolfont(int fontnumber)
+{
+   if (!strcmp(fonts[fontnumber].family, "Symbol")) return True;
+   return False;
+}
+
+/*----------------------------------------------------------------------*/
+/* Test equivalence of two label strings (relaxed constraints)		*/
+/*    Like stringcomp(), but ignores "superficial" differences such as  */
+/*    color and font (unless one of the fonts is Symbol), scale,	*/
+/*    underlining, tabbing, and kerning.				*/
+/*----------------------------------------------------------------------*/
+
+int stringcomprelaxed(stringpart *string1, stringpart *string2)
+{
+   stringpart *strptr1 = string1, *strptr2 = string2;
+   Boolean font1 = False, font2 = False;
+
+   if (strptr1->type == FONT_NAME)
+      font1 = issymbolfont(strptr1->data.font);
+   if (strptr2->type == FONT_NAME)
+      font2 = issymbolfont(strptr2->data.font);
+
+   while ((strptr1 != NULL) || (strptr2 != NULL)) {
+      while (strptr1 != NULL && strptr1->type != TEXT_STRING &&
+		strptr1->type != OVERLINE) {
+	 if (strptr1->type == FONT_NAME)
+	    font1 = issymbolfont(strptr1->data.font);
+	 strptr1 = strptr1->nextpart;
+      }
+      while (strptr2 != NULL && strptr2->type != TEXT_STRING &&
+		strptr2->type != OVERLINE) {
+	 if (strptr2->type == FONT_NAME)
+	    font2 = issymbolfont(strptr2->data.font);
+	 strptr2 = strptr2->nextpart;
+      }
+      if (strptr1 == NULL || strptr2 == NULL) break;
+      if (font1 != font2) return 1;
+      if (strptr1->type != strptr2->type) return 1;
+      else {
+         switch (strptr1->type) {
+	    case TEXT_STRING:
+	       if (strcmp(strptr1->data.string, strptr2->data.string)) return 1;
+	       break;
+	    case OVERLINE:
+	       if (strptr1->type != strptr2->type) return 1;
+	       break;
+         }
+	 strptr1 = strptr1->nextpart;
+	 strptr2 = strptr2->nextpart;
+      }
+   }
+
+   /* One string continues after the other ends. . . */ 
+   if (strptr1 != NULL || strptr2 != NULL) return 1;
+   return 0;
+}
 /*----------------------------------------------------------------------*/
 /* Find the number of parts in a string	(excluding parameter contents)	*/
 /*----------------------------------------------------------------------*/
@@ -695,7 +760,7 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
 }
 
 /*----------------------------------------------------------------------*/
-/* Draw and entire string, including parameter substitutions		*/
+/* Draw an entire string, including parameter substitutions		*/
 /*----------------------------------------------------------------------*/
 
 void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
@@ -705,8 +770,8 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
    short  fstyle, ffont, tmpjust, baseline;
    int    paramno, pos, group = 0;
    int    defaultcolor, curcolor, scolor;
-   short  oldx, oldy, oldfont, oldstyle;
-   float  tmpscale = 1.0;
+   short  oldx, oldfont, oldstyle;
+   float  tmpscale = 1.0, natscale = 1.0;
    float  tmpthick = xobjs.pagelist[areastruct.page]->wirewidth;
    XPoint newpoint, bboxin[2], bboxout[2];
    Boolean xm, ym;
@@ -767,7 +832,6 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 #endif
 
    oldx = newpoint.x;
-   oldy = newpoint.y;
    baseline = newpoint.y;
 
    /* do quick calculation on bounding box; don't draw if off-screen */
@@ -799,7 +863,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 	     case FONT_NAME:
 		ffont = strptr->data.font;
 		fstyle = 0;		   /* style reset by font change */
-	        if (oldy == newpoint.y) {  /* set top-level font and style */
+	        if (baseline == newpoint.y) {  /* set top-level font and style */
 	           oldfont = ffont;
 	           oldstyle = fstyle;
 	        }
@@ -811,7 +875,9 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		break;
 
 	     case FONT_SCALE:
-		tmpscale = strptr->data.scale;
+		tmpscale = natscale * strptr->data.scale;
+	        if (baseline == newpoint.y) /* reset top-level scale */
+		   natscale = tmpscale;
 		break;
 
 	     case KERN:
@@ -858,26 +924,29 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		break;
 
 	     case RETURN:
-		baseline -= BASELINE * tmpscale;
+		tmpscale = natscale = 1.0;
+		baseline -= BASELINE;
 	        newpoint.y = baseline;
 		newpoint.x = oldx;
 		break;
 	
 	     case SUBSCRIPT:
-	        tmpscale *= SUBSCALE; 
-	        newpoint.y -= (short)((TEXTHEIGHT >> 1) * tmpscale);
+	        natscale *= SUBSCALE; 
+		tmpscale = natscale;
+	        newpoint.y -= (short)((TEXTHEIGHT >> 1) * natscale);
 		break;
 
 	     case SUPERSCRIPT:
-	        tmpscale *= SUBSCALE;
-	        newpoint.y += (short)(TEXTHEIGHT * tmpscale);
+	        natscale *= SUBSCALE;
+		tmpscale = natscale;
+	        newpoint.y += (short)(TEXTHEIGHT * natscale);
 		break;
 
 	     case NORMALSCRIPT:
-	        tmpscale = 1.0;
+	        tmpscale = natscale = 1.0;
 	        ffont = oldfont;	/* revert to top-level font and style */
 	        fstyle = oldstyle;
-	        newpoint.y = oldy;
+	        newpoint.y = baseline;
 		break;
 
 	     case UNDERLINE:
@@ -904,9 +973,13 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		break;
 
 	     case HALFSPACE: case QTRSPACE: {
-		short addx = UDrawChar(localdata, (u_char)32, fstyle, ffont, group,
+		short addx;
+		UPushCTM();
+		UPreMultCTM(DCTM, newpoint, tmpscale, 0);
+		addx = UDrawChar(localdata, (u_char)32, fstyle, ffont, group,
 			curcolor);
 		newpoint.x += addx >> ((strptr->type == HALFSPACE) ? 1 : 2);
+		UPopCTM();
 		} break;
 
 	     case TEXT_STRING:
@@ -959,7 +1032,7 @@ enddraw:
 TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	short dostop, XPoint *tbreak)
 {
-   float oldscale, strscale, locscale = 1.0, xtotal = 0.5;
+   float oldscale, strscale, natscale, locscale = 1.0, xtotal = 0.5;
    float lasttotal = xtotal;
    stringpart *strptr;
    u_char *textptr;
@@ -973,28 +1046,32 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
    retext.width = retext.ascent = retext.descent = retext.base = 0;
    if (fontcount == 0) return retext;
 
-   if (newscale > 0.0) strscale = UTopScale() * newscale;
-   else strscale = 1.0;
+   if (newscale > 0.0) natscale = UTopScale() * newscale;
+   else natscale = 1.0;
      
-   oldscale = strscale;
+   oldscale = strscale = natscale;
 
    for (strptr = string; strptr != NULL;
 		strptr = nextstringpart(strptr, localdata)) {
       switch (strptr->type) {
 	 case SUPERSCRIPT:
-	    strscale *= SUBSCALE;
-	    ykern += TEXTHEIGHT * strscale / 2.0;
+	    natscale *= SUBSCALE;
+	    strscale = natscale;
+	    ykern += TEXTHEIGHT * natscale;
 	    break;
 	 case SUBSCRIPT:
-	    strscale *= SUBSCALE;
-	    ykern -= TEXTHEIGHT * strscale / 2.0;
+	    natscale *= SUBSCALE;
+	    strscale = natscale;
+	    ykern -= TEXTHEIGHT * natscale / 2.0;
 	    break;
 	 case NORMALSCRIPT:
-	    strscale = oldscale;
+	    natscale = strscale = oldscale;
 	    ykern = 0.0;
 	    break;
 	 case RETURN:
-	    retext.base -= BASELINE * strscale;
+	    natscale = strscale = oldscale;
+	    ykern = 0.0;
+	    retext.base -= BASELINE;
 	    if (!dostop)
 	       retext.width = max(retext.width, xtotal);
 	    xtotal = 0.5;
@@ -1002,12 +1079,12 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	 case HALFSPACE: 
 	    chptr = (*(somebet + 32));
 	    xtotal += (float)(chptr->width + chptr->lowerleft.x)
-			* locscale * strscale / 2;
+			* locscale * natscale / 2;
 	    break;
 	 case QTRSPACE:
 	    chptr = (*(somebet + 32));
 	    xtotal += (float)(chptr->width + chptr->lowerleft.x) 
-			* locscale * strscale / 4;
+			* locscale * natscale / 4;
 	    break;
 	 case TABBACKWARD:	/* find first tab value with x < xtotal */
 	    for (tabno = numtabs - 1; tabno >= 0; tabno--) {
@@ -1032,7 +1109,9 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	    tabstops[numtabs - 1] = xtotal;
 	    break;
 	 case FONT_SCALE:
-	    strscale = strptr->data.scale;
+	    strscale = natscale * strptr->data.scale;
+	    if (ykern == 0.0)
+	       natscale = strscale;
    	    if (newscale > 0.0)
 	       strscale *= UTopScale();
 	    break;
@@ -1043,6 +1122,8 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	 case FONT_NAME:
 	    somebet = fonts[strptr->data.font].encoding;
 	    locscale = fonts[strptr->data.font].scale;
+	    if (ykern == 0.0)
+	       natscale = locscale;
 	    break;
 	 case TEXT_STRING:
 	    for (textptr = strptr->data.string; *textptr != '\0'; textptr++) {
